;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x49684a76a2bbb5401960363921a12da9 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x54c46c66094c3e4fd68155d7bdb4e1ab . "./code.cg2js/cg-printing.fasl")
(#xb879fcdac69e733068c8f089d2ea4a18 . "./code.cg2js/cg-caret.fasl")
(#xd0f77326c5e9c767234b3c7547913888 . "./code.cg2js/cg-item-list.fasl")
(#xac160939e80ad7157235d886bf2dd524 . "./code.cg2js/cg-progress-indicator.fasl")
(#xacf2afe59d99b0b87354509afcbdf0c6 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc645a4d3b8183aefad835f2dae4e08b8 . "./code.cg2js/cg-outline.fasl")
(#x07fa525039248dd077b530fe963c5988 . "./code.cg2js/cg-palette.fasl")
(#x901bc90841cf534684c95ee4524f1c13 . "./code.cg2js/cg-selected-object.fasl")
(#x4639b3e86f15cfbdaef37ee993ffebf9 . "./code.cg2js/cg-status-bar.fasl")
(#xc2fb150bbcdc24e87214a6b559ebf624 . "./code.cg2js/cg-wrap-string.fasl")
(#xf6ad93c9ed68b9346881fe98c8b01d90 . "./code.cg2js/cg-fill-texture.fasl")
(#xdb0a2816ac5c4499cb9e370476d760a0 . "./code.cg2js/cg-layout.fasl")
(#x155365a580461ad8338466666fdc1f90 . "./code.cg2js/cg-timer.fasl")
(#x6d571d6aa645481433f16264ea55926c . "./code.cg2js/cg-dialog-item.fasl")
(#x26e9c3406460a90b7fc7213b410a404f . "./code.cg2js/cg-button.fasl")
(#x66b14a8754826c9a001a360ae9015ab3 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x971d719df14d885123cef3376220bd85 . "./code.cg2js/cg-tray.fasl")
(#x7aa50ae5bd36703f3f5932658a5d387c . "./code.cg2js/cg-plot-widget.fasl")
(#x2d2b7269ad8f8930c6d3dfe27f4a2e48 . "./code.cg2js/cg-pixmap.fasl")
(#x04026dd5514ee48b51b0010d3e82c3b0 . "./code.cg2js/cg-font-dialog.fasl")
(#x6a3a9186a37e2c97b0fb342bdb059464 . "./code.cg2js/cg-os-widget.fasl")
(#xe2178da71882a912efa31ab03ba14c1e . "./code.cg2js/cg-text-or-combo.fasl")
(#xf9ddb35372234eb80595e695905ceef2 . "./code.cg2js/cg-trackbar.fasl")
(#x09d6407971efae8ee0a0287be5cb5ef7 . "./code.cg2js/cg-up-down-control.fasl")
(#x1dd2eaa00650c27ab9de285cfbcdf81b . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x26b135670f584a3a38df1e98584e0111 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xae87b4aafd1f280f393e65cb1ef8abde . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9d25e9ca25782484741fc7c5589039bb . "./code.cg2js/cg-bitmap-stream.fasl")
(#x76d93f45f499889e4e598f309357211a . "./code.cg2js/cg-chart-widget.fasl")
(#x0680ce8218d58c068a36738ea160df14 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x260cff7ef8f9800b5ac1a0a361c1cd6e . "./code.cg2js/cg-group-box.fasl")
(#xe4190e1ade479b9c1fbfb30b1517bb2d . "./code.cg2js/cg-static-text.fasl")
(#xfc56b8c1d2756bdb4ada7559deb96742 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x81e3c85363631d53dbbb75c68671a374 . "./code.cg2js/cg-color-dialog.fasl")
(#xc08f39fdf16526055b3db459336a0db0 . "./code.cg2js/cg-intersect.fasl")
(#xc4552b74111c6c78ace5f17ca3cdba8c . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc212d648e1728c2d9641b7db29db2aee . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x66c8803c31705b0effd8a9a6f8f62b92 . "./code.cg2js/cg-html-dialog.fasl")
(#xeeef51288b1f8f2b39879dbc621188fa . "./code.cg2js/cg-lettered-menu.fasl")
(#x92b26b28b6350a1d2e3242ae2da3dbad . "./code.cg2js/cg-class-grid.fasl")
(#x55a6b5d5f7445e4bfae9e1ae2b4eb096 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x690a18e397884bf7dba0a8447a4a6d43 . "./code.cg2js/cg-calendar.fasl")
(#xa18dd06e49b76ac4caf3b9164a15d6f5 . "./code.cg2js/cg-editable-text.fasl")
(#x1e205badcffd71637589ff7a787cb133 . "./code.cg2js/cg-property.fasl")
(#x89d69f7663038c13e716b5ecd42e54ad . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xa1f73c05a8fc8c4a9ed81943eba9034e . "./code.cg2js/cg-grid-widget.fasl")
(#xab09cf2433cb31fc2a711a9082264e1f . "./code.cg2js/cg-paren-matching.fasl")
(#xd9ff3954dae947a453f1de58d5e07df9 . "./code.cg2js/cg-dev.fasl")
(#x8e4c91ba804c21a0e8f64cfc4b87dec0 . "./code.cg2js/cg-drawable.fasl")
(#x8614272e2bde48352c1876636db2ee15 . "./code.cg2js/cg-curve.fasl")
(#x3285285638701a9f2cf927f61248c749 . "./code.cg2js/cg-lisp-widget.fasl")
(#xdc37fabf6d1929593f6c4f970bfa294a . "./code.cg2js/cg-dde-utils.fasl")
(#xab2c423c6531d56b5a16578381273bba . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x1a6a651dab622d39cc9eb6588ca86b28 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x049cf99faa1255c2da3e06763ae7965f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x0a8c676157b1ca4eb59fcfc69dee4b33 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x96d66ece8ea5a0dcb40508d05c66d07e . "./code.cg2js/cg-lamp.fasl")
(#x07cee0c5f5d195f61bd5ea24116aafd3 . "./code.cg2js/cg-common-control.fasl")
(#xb56e4ccccc579bfb2d27824a07813813 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x01f59d9a89963af371feb0b4ea7288ac . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x28a508e9c0eab5c616e343d1555e1db6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd7e69af2edecd2b607d28bf59ad60469 . "./code.cg2js/cg-project-window.fasl")
(#x18fd5309470ab02ad581f57f4ececb7c . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe19a94cf2a14259c6d19c82754cbf453 . "./code.cg2js/cg-class-support.fasl")
(#x29842cbd910f99fd4537561ad355d79c . "./code.cg2js/cg-multi-picture-button.fasl")
(#x9f85c165f8501eb026f15de0d8b7c8b8 . "./code.cg2js/cg-yes-no-list.fasl")
(#x99d0d5de64de4e60689f40b52340baae . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x82f8edf0920acf64374658c22bef758b . "./code.cg2js/cg-string-dialog.fasl")
(#x1051e93208b37298f57a74f7870438c5 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9de7d600b04c431498da2c4a87b13904 . "./code.cg2js/cg-dropping-outline.fasl")
(#x6fb1913b45ce0561ac0b581fa1ac06b7 . "./code.cg2js/cg.fasl")
(#x753fa6bd510359a09dc3714be693ccd7 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4f3d37afd16f99afc2c43253bbb4f73e . "./code.cg2js/cg-scaling-stream.fasl")
(#x472ea5036801fd2d834166db7ff7ff94 . "./code.cg2js/cg-yes-no-string.fasl")
(#x80944f730bc820ab16de8c1726b1c854 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa88a1c1fa0b1b307c8aa41e200f1b1ba . "./code.cg2js/cg-web-browser.fasl")
(#xd0df3079e7ce597846cbbba6ea7c3716 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xba8ca2fd939ed0ccba56d0b2e93a56f1 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb62d1d75afc4dd03b430e1e96fc70e0d . "./code.cg2js/cg-message-dialog.fasl")
(#x7e4398347d59609a4ba0b13ad94de5d9 . "./code.cg2js/cg-comtab.fasl")
(#x1fde22c8f877a8233822dfc9dfe0b19e . "./code.cg2js/cg-base.fasl")
(#x0faa733a6f45967e65c39063f1866ce0 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xb24e062c504b19838e9b0b27aa56200e . "./code.cg2js/cg-text-edit-pane.fasl")
(#xc71c021aea03d510dfcee10a5db4f533 . "./code.cg2js/cg-utility-dialog.fasl")
(#xf29a4b4888af9d8f2e376997d9a0da84 . "./code.cg2js/cg-object-editor.fasl")
(#x9cd7a744d31bdd7f72aa20919bfe0f48 . "./code.cg2js/cg-edit-in-place.fasl")
(#x132f9e07b3169aa9a42d99b643ff6bfb . "./code.cg2js/cg-radio-button.fasl")
(#xe07ae2fc45256a51592ceac17b5fcd27 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x47185b164f5ea1f7a7f36e17f9bb2145 . "./code.cg2js/cg-nodes.fasl")
(#x5f542029ae39593134563eea781d1cc2 . "./code.cg2js/cg-get-position.fasl")
(#xc2910fadf86a52ce890f050ff0869a5a . "./code.cg2js/cg-directory-dialog.fasl")
(#x58a098a5d5abea21d2ae60e100d46ae1 . "./code.cg2js/cg-lisp-text.fasl")
(#xe3f5812b958cf50af4a45cab1d9277b1 . "./code.cg2js/cg-combo-box.fasl")
(#x9185dded2068d14889dcf43a66262157 . "./code.cg2js/cg-text-widget.fasl")
(#x944781b35d01ee27910199b0357197fd . "./code.cg2js/cg-list-view.fasl")
(#xda51f5971f266407f3fe6170fafc19da . "./code.cg2js/cg-graphics-context.fasl")
(#x8dd474edc164eb70536671802c66f629 . "./code.cg2js/cg-get-pixmap.fasl")
(#x5707f9b1204607b5b0a53ed95c70f76e . "./code.cg2js/cg-menu.fasl")
(#x4f81f3c5f286b714498694d2d00ba1f1 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe2f5b17f7a16312a6d88797e7548413a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbb8f06755e9938b08dd935499b376c90 . "./code.cg2js/cg-tooltip.fasl")
(#x18cf2f921fb38dc2fbeba1c088924baf . "./code.cg2js/cg-html-widget.fasl")
(#x4072f9346b17323c36517146dab8550a . "./code.cg2js/cg-choice-list.fasl")
(#x0b13d160bbcd0e87edcbb573e2d69238 . "./code.cg2js/cg-icon.fasl")
(#x984b3826f180b1006f0e0dc453394969 . "./code.cg2js/cg-acache.fasl")
(#x8c39d6e0fa926e06450e3986cacbe5ed . "./code.cg2js/cg-template-string.fasl")
(#x69dbdaadc1f60fc99e8e9d3f4880863c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x855d3e6ff7e95c0c6dc5e2f04ccd859f . "./code.cg2js/cg-toggling-widget.fasl")
(#x5062af1a360585a692095aeda582861f . "./code.cg2js/cg-tab-control.fasl")
(#x9d30982f07ed01602bca3d6178e58429 . "./code.cg2js/cg-split-bar.fasl")
(#x3f3b13a7e519e925117befdf473eac5a . "./code.cg2js/cg-picture-widget.fasl")
(#xcda08868efbbe063683ed89dac5d3317 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x541ee6f4c4545187c91b179e751d7b37 . "./code.cg2js/cg-choose-printer.fasl")
(#x4fdcd8564be7a4792d4311e508a39f47 . "./code.cg2js/cg-os-window.fasl")
(#xf94c1bb6372005a5badae7a058e24489 . "./code.cg2js/cg-scroll-bar.fasl")
(#x433d737fab819b0d5130c296c8fb94b7 . "./code.cg2js/cg-clipboard.fasl")
(#xf1a65df771c834325fb146f8fc1248b6 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x13d17a80a9727ae1e4914919d3d63bce . "./code.cg2js/cg-hotspot.fasl")
(#x2135a04389d95a90b7dcb83ac75bb215 . "./code.cg2js/cg-check-box.fasl")
(#x891b22e14761a87582489f845d44a2c7 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd56d13ad47940eb554b0da60af36efcb . "./code.cg2js/cg-file-dialog.fasl")
(#x178e1050c844c57471ed614a88f767b8 . "./code.cg2js/cg-toolbar.fasl")
(#x3c2f064b89ab1e581546b0dea0792c85 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xba22659c34c519ab0cc8a468c8b8a900 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x38d3002c52416a3ceafccfc9ddef3d8d . "./code.cg2js/cg-shortcut-menu.fasl")
(#xafdc7da23c58bb02dbb0780ff1438fff . "./code.idejs/ide-project.fasl")
(#x341dc7d463801264c03ae637ad9f4c5c . "./code.idejs/ide-intro.fasl")
(#x54123edf990db62dfa2a09ae0942dc8b . "./code.idejs/ide-find-in-files.fasl")
(#x68e988ee9689d0e16c0a06fddd47feb5 . "./code.idejs/ide-grapher.fasl")
(#x57f1224643f2e743c0ebfea3a0c42a5f . "./code.idejs/ide-inspector.fasl")
(#x0c897b61e53d18f50931d6bac2f9dfe7 . "./code.idejs/ide-editor.fasl")
(#x2b5f51aea9d5224ec16bb18943897e8a . "./code.idejs/ide-window-list.fasl")
(#x0640116ec4181118c35e0bcd530f4ab9 . "./code.idejs/ide-process-browser.fasl")
(#xa2b0e55b21a36eabca2c580455dae6d2 . "./code.idejs/ide-patch-dialog.fasl")
(#xeb96c3edb415e50c0d51bed0baba8e18 . "./code.idejs/ide-class-grapher.fasl")
(#x170549cd6bfcc751e58b692b66c171f6 . "./code.idejs/ide-base.fasl")
(#x0dc39949fe51f8c962c53708e629d791 . "./code.idejs/ide.fasl")
(#x0776e56c754c0c68e72bb8c7923581c8 . "./code.idejs/ide-trace.fasl")
(#x8b405a1ecbdf503ef5c7ef3b9378a74b . "./code.idejs/ide-form.fasl")
(#xdd4fc9fd24db2995e1ce4ce7c78d3a05 . "./code.idejs/ide-find-definitions.fasl")
(#xfc5f54371633c1e39b4749fa60b82b29 . "./code.idejs/ide-apropos.fasl")
(#x0658da910f02cb3f00a7cb8af7b40b42 . "./code.idejs/ide-class-browser.fasl")
(#x6cb9d77264577bf8331176695c3e2b56 . "./code.idejs/ide-navigator.fasl")
(#x405381546b95ef93172e430e30b3c0a4 . "./code.idejs/ide-clipboard.fasl")
(#x4cf8c9150261f7975a6c218cc9736d4b . "./code.idejs/ide-acache.fasl")
(#x01f80d16945cae127533af0defbc6394 . "./code.idejs/ide-keyshort.fasl")
(#x8399f5fe406d87295ac409e4e584910a . "./code.idejs/ide-package-list.fasl")
(#xae7d5fac3846c0e90dcd60be1a8b3ccd . "./code.idejs/ide-dev.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
